################################################################################
# Replication file for 'Stability and Change in the Freedom House Political    #
#    Rights and Civil Liberties Measures'  									   #
#                                                                              #
# This file post-processes the markov chains from the JAGS models to create    #
# all of the results, tables and figures from the article and appendix         #
################################################################################

# TABLE OF CONTENTS (line numbers of this file indicated) #

###### Print Manuscript ###### 
# Factor Model Convergence ........ 29-68
# Table II ........................ 71-165
# Table III ....................... 169-272
# Table IV ........................ 274-343
# Figure 1 ........................ 345-355
# Figure 2 ........................ 357-421
# Figure 3 ........................ 423-551
# Figure 4 ........................ 553-619

###### On-line Appendix ###### 
# Latent Class Model Convergence .. 626-653
# Table A I ....................... 655-687
# Figure A1 ....................... 689-744
# Figure A2 ....................... 746-777


##################
## Factor Model ##
##################

## Read in CODA files from the Factor Model
chain1 <- read.coda("fh_prcl_chain1.txt", "fh_prcl_index.txt")
chain2 <- read.coda("fh_prcl_chain2.txt", "fh_prcl_index.txt")

## Assess convergence

# take out non-stochastic nodes
vars <- apply(chain1, 2, var)
diag.chain1 <- chain1[, which(vars > 0)]
diag.chain2 <- chain2[, which(vars > 0)]

# perform Gelman and Rubin diagnostic 
gel <- gelman.diag(mcmc.list(diag.chain1, diag.chain2))

# make traceplots of monitored chain values
par(mfrow=c(3,2), ask=T)
traceplot(mcmc.list(diag.chain1, diag.chain2))

# make density plots of monitored chain values
par(mfrow=c(3,2), ask=T)
densplot(mcmc.list(diag.chain1, diag.chain2))

## Process chain values to extract latent point estimates

# makes new objects of just PR and CL chain values
chains <- rbind(chain1, chain2)
latent.pr <- chains[,grep("latent.polrts", colnames(chains))]
latent.cl <- chains[,grep("latent.civlibs", colnames(chains))]

# create country-time variable on which to match up the JAGS output to the existing data
ct <- paste(fh$ccode, fh$time, sep=".")
latent.ct <- gsub(",", ".", gsub("]", "", sapply(strsplit(colnames(latent.pr), split="[", fixed=T), function(x)x[2]), fixed=T), fixed=T)

# re-order the latent PR and CL values so the columns of the MCMC output are the same as the rows of the original dataset
new.pr <- latent.pr[, match(ct, latent.ct)]
new.cl <- latent.cl[, match(ct, latent.ct)]


##############
## Table II ##
##############
## Note, Table I is not a statistical table, thus no results are required to produce Table I

## Find significant differences within the same FH PR classification

# initialize the matrices of comparison probabilities
pr.diff.mat <- cl.diff.mat <- matrix(ncol=7, nrow=7)

# loop over the 7 original categories of political rights
for(j in 1:7){
	# grab only those chain values for the original observations in category j
	test <- new.pr[, which(fh$PR == j)]
	# create a list of all possible combinations of size 2 for the columns of test
	combs <- t(combn(ncol(test),2))
	# initialize a matrix that will implement the pairwise comparisons of each combination
	diffmat <- matrix(0, nrow=ncol(test), ncol=nrow(combs))
	# put ones and negative ones in for the relevant entries to allow taking the pairwise difference
	diffmat[cbind(combs[,1], 1:nrow(combs))] <- 1
	diffmat[cbind(combs[,2], 1:nrow(combs))] <- -1
	# create the pairwise differences
	diffs <- test %*% diffmat
	# calculate the probability that each difference is bigger than zero
	diffs.p <- apply(diffs, 2, function(x)mean(x > 0))
	# replce the j^{th} diagonal element of pr.diff.mat with the proportion of observations where the difference is one-tailed significant at .95
 	pr.diff.mat[j,j] <- mean(diffs.p > .95 | diffs.p < .05)
	# print out j so you know where the function is
	cat(j, " ", sep="")
}

# the command below does the same thing as the previous command, but for civil liberties, thus comments are omitted 
for(j in 1:7){
	test <- new.cl[, which(fh$CL == j)]
	combs <- t(combn(ncol(test),2))
	diffmat <- matrix(0, nrow=ncol(test), ncol=nrow(combs))
	diffmat[cbind(combs[,1], 1:nrow(combs))] <- 1
	diffmat[cbind(combs[,2], 1:nrow(combs))] <- -1
	diffs <- test %*% diffmat
	diffs.p <- apply(diffs, 2, function(x)mean(x > 0))
	cl.diff.mat[j,j] <- mean(diffs.p > .95 | diffs.p < .05)
	cat(j, " ", sep="")
}

# produce all of the combinations necessary to compare the difference of PR latents across the original PR categories
meta.comb <- t(combn(7,2))

# loop over all combinations
for(j in 1:nrow(meta.comb)){
	# pull out those values related to the first partner in the combination
	mat1 <- new.pr[, which(fh$PR == meta.comb[j,1])]
	# pull out those values related to the second partner in the combination
	mat2 <- new.pr[, which(fh$PR == meta.comb[j,2])]
	# bind the two matrices together
	mat.12 <- cbind(mat1, mat2)

	# create all of the pairwise combinations for every column of mat.12
	com12 <- t(combn(ncol(mat.12), 2))
	# remove combinations among the columns of the first matrix and among the columns of the second matrix
	com12 <- com12[-which(com12[,2] <= ncol(mat1)),]
	com12 <- com12[-which(com12[,1] > ncol(mat1)),]

	# create the difference matrix to implement the pairwise comparisons across the two categories
	diffmat <- matrix(0, nrow=ncol(mat.12), ncol=nrow(com12))
	# put ones and negative ones in for the relevant entries to allow taking the pairwise difference
	diffmat[cbind(com12[,1], 1:nrow(com12))] <- 1
	diffmat[cbind(com12[,2], 1:nrow(com12))] <- -1
	# calculate the pairwise differences across the categories
	diffs <- mat.12 %*% diffmat
	# for each combination, calculate the probability that the difference is bigger than zero
	diffs.p <- apply(diffs, 2, function(x)mean(x > 0))
	# fill in the appropriate element of the pr.diff.mat matrix with the proportion of observations for which the differences were significant
	pr.diff.mat[meta.comb[j,2], meta.comb[j,1]] <- mean(diffs.p > .95 | diffs.p < .05)
	cat(j, " ", sep="")
}


# do the same as above, but for civil liberties (comments omitted)
for(j in 1:nrow(meta.comb)){
	mat1 <- new.cl[, which(fh$CL == meta.comb[j,1])]
	mat2 <- new.cl[, which(fh$CL == meta.comb[j,2])]
	mat.12 <- cbind(mat1, mat2)

	com12 <- t(combn(ncol(mat.12), 2))
	com12 <- com12[-which(com12[,2] <= ncol(mat1)),]
	com12 <- com12[-which(com12[,1] > ncol(mat1)),]

	diffmat <- matrix(0, nrow=ncol(mat.12), ncol=nrow(com12))
	diffmat[cbind(com12[,1], 1:nrow(com12))] <- 1
	diffmat[cbind(com12[,2], 1:nrow(com12))] <- -1
	diffs <- mat.12 %*% diffmat
	diffs.p <- apply(diffs, 2, function(x)mean(x > 0))
	cl.diff.mat[meta.comb[j,2], meta.comb[j,1]] <- mean(diffs.p > .95 | diffs.p < .05)
	cat(j, " ", sep="")
}



###############
## Table III ##
###############

## Find out how many significant over-time differences there are in Political Rights latent variables

# initialize results vectors
pr.pos.diffs <- pr.neg.diffs <- rep(NA, 193)

# loop over the 193 countries in the dataset
for(j in 1:193){
	# find the values of the Political Rights latent variable chains for country j
	test <- new.pr[, which(fh$ccode == j)]
	# create a list of all possible combinations of size 2 for the columns of test
	combs <- t(combn(ncol(test),2))
	# initialize a matrix that will implement the pairwise comparisons of each combination
	diffmat <- matrix(0, nrow=ncol(test), ncol=nrow(combs))
	# put ones and negative ones in for the relevant entries to allow taking the pairwise difference
	diffmat[cbind(combs[,1], 1:nrow(combs))] <- 1
	diffmat[cbind(combs[,2], 1:nrow(combs))] <- -1
	# create the pairwise differences
	diffs <- test %*% diffmat
	# calculate the probability that each difference is bigger than zero
	diffs.p <- apply(diffs, 2, function(x)mean(x > 0))
	# replace the j^{th} element of pr.pos.diffs with 1 if the country experienced a significant positive change
	pr.pos.diffs[j] <- max(diffs.p < 0.05) 
	# replace the j^{th} element of pr.neg.diffs with 1 if the country experienced a significant negative change
	pr.neg.diffs[j] <- max(diffs.p > .95)
	# print j to show the iteration
	cat(j, " ", sep="")
}

# do the same thing as above for Civil Liberties (comments omitted)
cl.pos.diffs <- cl.neg.diffs <- rep(NA, 193)
for(j in 1:193){
	test <- new.cl[, which(fh$ccode == j)]
	combs <- t(combn(ncol(test),2))
	diffmat <- matrix(0, nrow=ncol(test), ncol=nrow(combs))
	diffmat[cbind(combs[,1], 1:nrow(combs))] <- 1
	diffmat[cbind(combs[,2], 1:nrow(combs))] <- -1
	diffs <- test %*% diffmat
	diffs.p <- apply(diffs, 2, function(x)mean(x > 0))
	cl.pos.diffs[j] <- max(diffs.p < 0.05)
	cl.neg.diffs[j] <- max(diffs.p > .95)
	cat(j, " ", sep="")
}


## Calculate the number of countries that have at least one change over time in their FH PR scores

# initialize results vectors
fh.pr.pos <- fh.pr.neg <- rep(NA, 193)
# loop over the 193 countries
for(j in 1:193){
	# find the values of PR for country j
	test <- fh$PR[which(fh$ccode == j)]
	# create a list of all possible combinations of size 2 for the columns of test
	combs <- t(combn(length(test),2))
	# Find the pairwise differences for all years in country j
	diffs <- test[combs[,1]] - test[combs[,2]]
	# replace fh.pr.neg equals 1 if the j^{th} country had a negative change in FH PR
	fh.pr.neg[j] <- sign(max(-diffs))
	# replace fh.pr.pos equals 1 if the j^{th} country had a positive change in FH PR
	fh.pr.pos[j] <- sign(max(diffs))
}

# do the same thing as above for Civil Liberties (comments omitted)
fh.cl.pos <- fh.cl.neg <- rep(NA, 193)
for(j in 1:193){
	test <- fh$CL[which(fh$ccode == j)]
	combs <- t(combn(length(test),2))
	diffs <- test[combs[,1]] - test[combs[,2]]
	fh.cl.neg[j] <- sign(max(-diffs))
	fh.cl.pos[j] <- sign(max(diffs))
}


# Create new variables to make Table III
fh.pr.diffs <- fh.pr.pos + 2*fh.pr.neg
fh.cl.diffs <- fh.cl.pos + 2*fh.cl.neg

pr.diffs <- pr.pos.diffs + 2*pr.neg.diffs
cl.diffs <- cl.pos.diffs + 2*cl.neg.diffs

# Make the new variables into factors
labs <- c("None", "Positive", "Negative", "Both")
fh.pr.diffs <- factor(fh.pr.diffs, levels=c(0,1,2,3), labels=labs)
fh.cl.diffs <- factor(fh.cl.diffs, levels=c(0,1,2,3), labels=labs)
pr.diffs <- factor(pr.diffs, levels=c(0,1,2,3), labels=labs)
cl.diffs <- factor(cl.diffs, levels=c(0,1,2,3), labels=labs)

# Make tables
pr.tab <- table(pr.diffs, fh.pr.diffs)
pr.tab
cl.tab <- table(cl.diffs, fh.cl.diffs)
cl.tab

## Assess the temporal and cross-sectional variance

# Pull out only those parameters related to the latent variable variances
sds <- sqrt(1/chains[, grep("tau.l", colnames(chains))])

# Find the median and 95% Credible Interval for the standard deviations. 
t(apply(sds, 2, quantile, c(.5,.025,.975)))

##############
## Table IV ##
##############

## Run the model of physical integrity rights on press freedom and the original PR variables
# note: in all models, the PR values are transformed using scale() to have mean 0 and unit variance.
mod1 <- lm(physint ~ fhpf*scale(PR), data=fh)

## Run the model of physical integrity rights on press freedom and the means of the posterior distribution
# for the PR latent variable (no measurement uncertainty incorporated)
mod2 <- lm(physint ~ fhpf*scale(-newpr), data=fh)

## Run the model of physical integrity rights on press freedom and the means of the posterior distribution
# for the PR latent variable (measurement uncertainty incorporated)

# First, we need to initialize a matrix that will hold our results
b <- array(dim=c(2500,6))

# loop over the 2500 monitored MC iterations
for(i in 1:2500){
	# for each iteration, estimate the model using a different row of the new.pr matrix
	mod <- lm(physint ~ fhpf*scale(-new.pr[i,]), data=fh)
	# in the i^{th} row of b, store one draw from MVN(b, V)
	b[i,] <- mvrnorm(1, mod$coef, vcov(mod))
	# set up the function to pring only if i is divisible by 25
	if(i %% 25 == 0){
		cat(i, " ", sep="")
	}
}

# To make conditional coefficients, set up a matrix A that will do the proper adding
A <- rbind(
	c(1,0,0,0,0,0), 
	c(1,1,0,0,0,0), 
	c(1,0,1,0,0,0), 
	c(0,0,0,1,0,0), 
	c(0,0,0,1,1,0), 
	c(0,0,0,1,0,1))

# Save the variance-covariance matrix of the estimators from mod1
V <- vcov(mod1)
# caluclate the standard errors of the conditional coefficients
se1 <- sqrt(diag(A%*%V%*%t(A)))
# save the coefficients from mod1
b1 <- coef(mod1)
# calculate the conditional coefficients from mod1
cond.b1 <- A %*% b1
# calculate the conditional t-statistics
t1 <-  cond.b1/se1
# calculate the two-sided p-values for the conditional coefficients
p1 <- round(2*pt(abs(t1), mod1$df.residual, lower.tail=F), 3)

# Does same as above, but for mod2 (comments omitted)
V <- vcov(mod2)
se2 <- sqrt(diag(A%*%V%*%t(A)))
b2 <- coef(mod2)
cond.b2 <- A %*% b2
t2 <-  cond.b2/se2
p2 <- round(2*pt(abs(t2), mod2$df.residual, lower.tail=F), 3)

# Calculate the conditional coefficients for each iteration of the MC for model 3
# the result is a 6x2500 matrix of conditional coefficients
cond.b3 <- A %*% t(b)
# calculate the standard error by taking the standard error of the MC values for each conditional coefficient
se3 <- apply(cond.b3, 1, sd)
# Find the temporary p-value by calculating the proportion of conditional coefficients that are less than zero. 
p3a <- apply(cond.b3, 1, function(x)mean(x < 0))
# calculate the p-value as 2x the smallest one-sided p-value. 
p3 <- sapply(p3a, function(x)2*min(x, 1-x))
cb3 <- apply(cond.b3, 1, mean)

##############
## Figure 1 ##
##############

library(MCMCpack)
s <- seq(0,25, length=1000)
pdf("igamma_pdf.pdf", height=6, width=6)
plot(dinvgamma(s, 1, 1) ~ s, type="l", xlab="x", ylab="Density", axes=F)
axis(1)
box()
dev.off()

##############
## Figure 2 ##
##############

## Read in the PWT population data
pop <- read.csv("jpr_population.csv")

# find 50 most populous countries
ccs <- pop$ccode[which(rank(-pop$POP) <= 50)]

## Create data frame for political rights with mean, lower and upper credible intervals
pr.dot <- as.data.frame(t(apply(new.pr[,ccs], 2, quantile, probs=c(.5,.025, .975))))

# add country name to dataset and sort on median political rights
pr.dot$country <- fh$Country[ccs]
pr.dot <- pr.dot[order(pr.dot[,1]), ]

# set attributes of data frame and variables
rownames(pr.dot) <- NULL
names(pr.dot) <- c("pr1", "pr.l1", "pr.u1", "country")
pr.dot$country <- as.factor(pr.dot$country)

# reorder the levels of the country factor according to the mean of political rights
pr.dot$country <- reorder(pr.dot$country, pr.dot$pr1)


## Create data frame for civil liberties with mean, lower and upper credible intervals
cl.dot <- as.data.frame(t(apply(new.cl[,ccs], 2, quantile, probs=c(.5,.025, .975))))

# add country name to data set and sort on median civil liberties
cl.dot$country <- fh$Country[ccs]
cl.dot <- cl.dot[order(cl.dot[,1]), ]

# set attributes of data frame and variables.
rownames(cl.dot) <- NULL
names(cl.dot) <- c("cl1", "cl.l1", "cl.u1", "country")
cl.dot$country <- as.factor(cl.dot$country)
cl.dot$country <- reorder(cl.dot$country, cl.dot$cl1)
names(cl.dot) <- names(pr.dot) <- c("latent", "latent.l1", "latent.u1", "country")


# put civil liberties and political rights datasets together
dot.fh <- rbind(pr.dot, cl.dot)

# order the country factor based on the median of political rights
reorder(dot.fh$country, c(dot.fh[1:50,1],dot.fh[1:50,1]))

# add a variable that will identify whether the entry refers to civil liberties or political rights
dot.fh$type <- c(rep(1, 50), rep(2,50))
dot.fh$type <- factor(dot.fh$type, levels=c(1,2), labels=c("Political Rights", "Civil Liberties"))

# reorder the dataset so the values print in the right order
dot.fh <- dot.fh[order(dot.fh$type, dot.fh$country), ]
dot.fh$ordering <- c(dot.fh[1:50,1], dot.fh[1:50,1])
dot.fh$country <- reorder(dot.fh$country, dot.fh$ordering)


# set parameters of the trellis display
trellis.par.set(list(axis.text=list(cex=.6), strip.background = list(col="gray90")))

# make dotplot of political rights and civil liberties
dotplot(country ~ latent | type, data=dot.fh, xlab="", xlim=c(min(dot.fh$latent.l1), max(dot.fh$latent.u1)), layout=c(2,1), panel=function(x,y,subscripts){
    panel.abline(v=0, col="red")
    panel.segments(dot.fh$latent.l1[subscripts], y, dot.fh$latent.u1[subscripts], y, col="gray50")
    panel.points(x,y, pch=16, cex=.6, col="black")})

##############
## Figure 3 ##
##############

## Add the mean of the posterior distributions of political rights and civil liberties to the dataset
fh$newpr <- apply(new.pr, 2, mean)
fh$newcl <- apply(new.cl, 2, mean)

# make two matrices where only the original FH scores and the new posterior means are kept
tmp.mat1 <- fh[,c("PR", "newpr")]
tmp.mat2 <- fh[,c("CL", "newcl")]

# change the column names of both temporary matrices
colnames(tmp.mat1) <- colnames(tmp.mat2) <- c("FH", "latent")

# make a new matrix from the two temporary matrices
dat2 <- rbind(tmp.mat1, tmp.mat2 )

# make a type variable that indicates whether the FH and posterior mean scores refer to PR or CL
dat2$type <- rep(c(1,2), each=772)
dat2$type <- factor(dat2$type, levels=c(1,2), labels=c("Political Rights", "Civil Liberties"))


## The panel.violin2 function is a slightly modified version of panel.violin which comes standard with 
## the lattice library.  I modified it to produce lines that cover the IQR in the middle of the violins
## and dots at the median.  

panel.violin2 <- function (x, y, box.ratio = 1, box.width = box.ratio/(1 + box.ratio), 
    horizontal = TRUE, alpha = plot.polygon$alpha, border = plot.polygon$border, 
    lty = plot.polygon$lty, lwd = plot.polygon$lwd, col = plot.polygon$col, 
    varwidth = FALSE, bw = NULL, adjust = NULL, kernel = NULL, 
    window = NULL, width = NULL, n = 50, from = NULL, to = NULL, 
    cut = NULL, na.rm = TRUE, cex=1, ...) 
{
    if (all(is.na(x) | is.na(y))) 
        return()
    x <- as.numeric(x)
    y <- as.numeric(y)
    plot.polygon <- trellis.par.get("plot.polygon")
    darg <- list()
    darg$bw <- bw
    darg$adjust <- adjust
    darg$kernel <- kernel
    darg$window <- window
    darg$width <- width
    darg$n <- n
    darg$from <- from
    darg$to <- to
    darg$cut <- cut
    darg$na.rm <- na.rm
    my.density <- function(x) {
        ans <- try(do.call("density", c(list(x = x), darg)), 
            silent = TRUE)
        if (inherits(ans, "try-error")) 
            list(x = rep(x[1], 3), y = c(0, 1, 0))
        else ans
    }
    numeric.list <- if (horizontal) 
        split(x, factor(y))
    else split(y, factor(x))
    levels.fos <- as.numeric(names(numeric.list))
    d.list <- lapply(numeric.list, my.density)
    dx.list <- lapply(d.list, "[[", "x")
    dy.list <- lapply(d.list, "[[", "y")
    max.d <- sapply(dy.list, max)
    if (varwidth) 
        max.d[] <- max(max.d)
    xscale <- current.panel.limits()$xlim
    yscale <- current.panel.limits()$ylim
    height <- box.width
	qx.list <- lapply(numeric.list, quantile, probs=c(.5,.25,.75))
    if (horizontal) {
        for (i in seq_along(levels.fos)) {
            if (is.finite(max.d[i])) {
                pushViewport(viewport(y = unit(levels.fos[i], 
                  "native"), height = unit(height, "native"), 
                  yscale = c(max.d[i] * c(-1, 1)), xscale = xscale))
                grid.polygon(x = c(dx.list[[i]], rev(dx.list[[i]])), 
                  y = c(dy.list[[i]], -rev(dy.list[[i]])), default.units = "native", 
                  gp = gpar(fill = col, col = border, lty = lty, 
                    lwd = lwd, alpha = alpha))
                popViewport()
                pushViewport(viewport(y = unit(levels.fos[i], 
                  "native"), height = unit(height, "native"), 
                  yscale = c(max.d[i] * c(-1, 1)), xscale = xscale))
				grid.segments(qx.list[[i]][2], dy.list[[i]][1], qx.list[[i]][3], dy.list[[i]][1], default.units="native")
                popViewport()
                pushViewport(viewport(y = unit(levels.fos[i], 
                  "native"), height = unit(height, "native"), 
                  yscale = c(max.d[i] * c(-1, 1)), xscale = xscale))
				grid.points(qx.list[[i]][1], dy.list[[i]][1], default.units="native", pch=16, gp=gpar(col="black", cex=cex))
                popViewport()
            }
        }
    }
    else {
        for (i in seq_along(levels.fos)) {
            if (is.finite(max.d[i])) {
                pushViewport(viewport(x = unit(levels.fos[i], 
                  "native"), width = unit(height, "native"), 
                  xscale = c(max.d[i] * c(-1, 1)), yscale = yscale))
                grid.polygon(y = c(dx.list[[i]], rev(dx.list[[i]])), 
                  x = c(dy.list[[i]], -rev(dy.list[[i]])), default.units = "native", 
                  gp = gpar(fill = col, col = border, lty = lty, 
                    lwd = lwd, alpha = alpha))
                popViewport()
                pushViewport(viewport(y = unit(levels.fos[i], 
                  "native"), height = unit(height, "native"), 
                  yscale = c(max.d[i] * c(-1, 1)), xscale = xscale))
				grid.segments(dy.list[[i]][1], qx.list[[i]][2], dy.list[[i]][1], qx.list[[i]][3], default.units="native")
                popViewport()
                pushViewport(viewport(y = unit(levels.fos[i], 
                  "native"), height = unit(height, "native"), 
                  yscale = c(max.d[i] * c(-1, 1)), xscale = xscale))
				grid.points(dy.list[[i]][1], qx.list[[i]][1], default.units="native", pch=16, gp=gpar(col="black", cex=cex))
                popViewport()
            }
        }
    }
    invisible()
}

# set some trellis parameters
trellis.par.set(list(strip.background = list(col="gray90")))

# make the violin plot
bwplot(FH ~ latent | type, data=dat2, xlab="Latent Variables", ylab="Freedom House Score", panel=function(x,y){
	panel.violin2(x,y,horizontal=T, col="white", cex=.35)
})

##############
## Figure 4 ##
##############

## Find the coefficients relating the latent variables to the observed PR and CL variables
b.pr <- chains[, grep("b.pr", colnames(chains))]
b.cl <- chains[, grep("b.cl", colnames(chains))]

# Create a matrix containing the posterior median and 95% credible intervals for each of the coefficients
coefs <- rbind(t(apply(b.pr, 2, quantile, probs=c(.5,.025,.975))), 
t(apply(b.cl,2, quantile, probs=c(.5,.025,.975))))

# make a data frame out of the coefs matrix
coefs <- as.data.frame(coefs)
# set the names attribute of the coefs data frame
names(coefs) <- c("median", "lower", "upper")

# Initialize and appropriately populate the year variable in the coefs data frame
coefs$year <- NA
coefs$year[grep("1,", rownames(coefs), fixed=T)] <- 2006
coefs$year[grep("2,", rownames(coefs), fixed=T)] <- 2007
coefs$year[grep("3,", rownames(coefs), fixed=T)] <- 2008
coefs$year[grep("4,", rownames(coefs), fixed=T)] <- 2009

# Identify the names of the seven issues
issues <- c("Electoral Process", "Political Pluralism", "Functioning of Govt", "Freedom of Expression", "Associational Rights", "Rule of Law", "Personal Autonomy")

# Initialize and appropriately populate the issue variable in the coefs data frame
coefs$issue <- ""
coefs$issue[1:4] <- issues[1]
coefs$issue[5:8] <- issues[2]
coefs$issue[9:12] <- issues[3]
coefs$issue[13:16] <- issues[4]
coefs$issue[17:20] <- issues[5]
coefs$issue[21:24] <- issues[6]
coefs$issue[25:28] <- issues[7]

# Make the prcl variable in the coefs data frame indicating whether the issues were PR or CL issues
coefs$prcl <- c(rep(1,  12), rep(2,16))

# make the prcl variable into afactor
coefs$prcl <- factor(coefs$prcl, levels=c(1,2), labels=c("Political Rights", "Civil Liberties"))

# Make a y variable that will serve as the y-axis variable for the dotplot
coefs$y <- c(34,33,32,31,29,28,27,26,24,23,22,21,19,18,17,16,14,13,12,11,9,8,7,6,4,3,2,1)

# find the mean value of y for each issue
ag <- aggregate(coefs$y, list(coefs$issue), mean)

# Initialize the plotting region
plot(c(min(coefs[,2]), max(coefs[,3])), c(1,34), type="n", axes=F, xlab="Coefficient", ylab="")
# Add the issues text
text(rep(mean(par()$usr[1:2]), 7), ag[,2], ag[,1], cex=1.75, col="gray60")
# Add the credible intervals
segments(coefs[,2], coefs$y, coefs[,3], coefs$y, col="black")
# Add the posterior medians
points(coefs[,1], coefs$y, pch=16, col="black", cex=.8)
# Add the x-axis ticks and labels
axis(1)
# Add the box around the interior plotting region
box()
# Place a horizontal line at y=20
abline(h=20, lty=2)
# Add the right-axis annotation
mtext(c("Political Rights", "Civil Liberties"), side=4, at=c(27.5, 10))
# Add the left-axis annotation
mtext(coefs$year, side=2, line=0.5, at=coefs$y, las=1, cex=.75)


###############################
## Results from the Appendix ##
###############################

########################
## Latent Class Model ##
########################

## Read in chains from JAGS model
chain1 <- read.coda("fh_lc_chain1.txt", "fh_lc_index.txt")
chain2 <- read.coda("fh_lc_chain2.txt", "fh_lc_index.txt")

## Assess convergence

# take out non-stochastic nodes
vars <- apply(chain1, 2, var)
diag.chain1 <- chain1[, which(vars > 0)]
diag.chain2 <- chain2[, which(vars > 0)]

# perform Gelman and Rubin diagnostic 
gel <- gelman.diag(mcmc.list(diag.chain1, diag.chain2))

# make traceplots of monitored chain values
par(mfrow=c(3,2), ask=T)
traceplot(mcmc.list(diag.chain1, diag.chain2))

# make density plots of monitored chain values
par(mfrow=c(3,2), ask=T)
densplot(mcmc.list(diag.chain1, diag.chain2))

# Make a single matrix of chain values
chains <- rbind(chain1, chain2)

###############
## Table A I ##
###############

## Create transition probabilities

## find the observation number for the lag of each observation 
# first, lag time by subtracting one
fh$lagtime <- fh$time - 1
# create a country-time variable
fh$ct <- paste(fh$Country, fh$time, sep=".")
# create a country-lag-time variable
fh$lct <- paste(fh$Country, fh$lagtime, sep=".")
# find the lagged observation number by using the match function
fh$lagobs <- match(fh$lct, fh$ct)

# freestat is the 2500x772 matrix of chain values
# create lagfreestat which is the lagged chain values
lagfreestat <- freestat[, fh$lagobs]
# For each MCMC iteration, tabulate the lagged and current values to get transition frequencies
laps <- lapply(1:2500, function(x)table(factor(lagfreestat[x,], levels=c(1,2,3)), 
	factor(freestat[x,], levels=c(1,2,3))))
# change transition frequencies into proportions with prop.table
laps2 <- lapply(laps, prop.table, margin=1)

# In each element of the laps2 list, turn the matrix into a vector
laps3 <- lapply(laps2, function(x)c(x))
# rbind together all of the transition probabilities
mat3 <- do.call(rbind, laps3)
# Find the posterior median, 2.75^{th}, and 97.5^{th} quantiles for each of the transition probabilities
trans.probs <- t(apply(mat3, 2, quantile, probs=c(.5,.025,.975)))
# calculate transition probabilities for the original FH data
fh.trans <- prop.table(table(fh$stat2[fh$lagobs], fh$stat2), 1)

###############
## Figure A1 ##
###############

# Create the appropriate FH Status variable
fh$numstat <- as.numeric(recode(fh$Status, "'F'=1; 'PF'=2; 'NF'=3"))
fh$stat2 <- ordered(fh$numstat, levels=c(1,2,3), labels=c("F", "PF", "NF"))

# Extract freedom status from latent class chains
freestat <- chains[, grep("free[", colnames(chains), fixed=T)]
# For each observation, tabulate the freedom status across the 2500 chain values
freetab <- t(apply(freestat, 2, function(x)table(factor(x, levels=c(1,2,3)))))
# Create a factor that identifies the freedom status of the mode of the posterior for each observation
free_lc <- ordered(apply(freetab, 1, which.max), levels=c(1,2,3), labels=c("F", "PF", "NF"))
# add free_lc to the dataset
fh$free_lc <- free_lc
		
# calculate the number of observations in each freedom status for both the lc model and the FH data
# for each time-period.  Make a single matrix called dist.fh of those values
dist.fh <- rbind(aggregate(rep(1, 772), list(fh$time, fh$free_lc), sum), aggregate(rep(1, 772), list(fh$time, fh$stat2), sum))

# Set the column names attribute of the dist.fh object
colnames(dist.fh) <- c("time", "status", "freq")

# Set the attributes of the variables we want to plot
dist.fh$time <- dist.fh$time + 2005
dist.fh$var <- factor(c(rep(1, 12),rep(2,12)), levels=c(1,2), labels=c("Latent Class", "Freedom House"))

# set some trellis parameters
trellis.par.set(list(strip.background = list(col="gray90")))

# Make the barchart for Figure A1
barchart(freq ~ var | as.factor(time), dist.fh, groups = status, stack = TRUE, horizozntal=F, ylab="Frequency",
	as.table=T, col=c("gray25", "gray50", "gray75"), ylim=c(0,196), 
	key=list(rectangles=list(col="gray25"), text=list("Free"), rectangles=list(col="gray50"), text=list("Partly Free"), 
		rectangles=list(col="gray75"), text=list("Not Free")))

## Analyze FH classification vs. the Latent Class model 

# For each iteration of the MC, find whether its predicted class equals the FH classification
aps <- apply(freestat, 1, function(x) x == fh$numstat)
# calculate the probability that each observation's LC membership is the same as the FH classification
aps2 <- apply(aps, 1, mean)
# find which observations have probabilities of less than 0.05
inds <- which(aps2 < .05)
# Make a cross-tabulation with my categorization in the rows and FH classification in the columns.
table(fh$free_lc[inds], fh$stat2[inds])

# Change freetab into row proportions
freetab2 <- prop.table(freetab, 1)
# find the maximum proportion in each row (probability of being in the posterior modal category)
max.ft2 <- apply(freetab2, 1, max)
# find the observations with probability greater than .95 of being in a single category
inds2 <- which(max.ft2 > .95)
# find the observations with probability less than .95 of being in a single category. 
inds2a <- which(max.ft2 < .95)

###############
## Figure A2 ##
###############

# Find the values associated with the parameters
lcparams <- chains[, grep("g", colnames(chains))]

# Calculate the mean and 95% Credible Interval for each parameter 
lcpdat <- t(apply(lcparams, 2, function(x)c(mean(x), quantile(x, c(.025,.975)))))
# Set the column names attribute of the lcpdat matrix
colnames(lcpdat) <- c("mean", "lower", "upper")
# Change the class of lcpdat to data.frame
lcpdat <- as.data.frame(lcpdat)
# Create the issue variable in lcpdat
lcpdat$issue <- issues[as.numeric(substr(rownames(lcpdat), start=2,stop=2))]
# Create the freedom status variable in lcpdat
lcpdat$status <- rep(c(1,2,3), 28)
# Make a factor variable out of the status variable in lcpdat
lcpdat$status <- ordered(lcpdat$status, levels=c(1,2,3), labels=c("F", "PF", "NF"))
# Create a sequence of years from 2006-09
years <- 2006:2009
# Create a year variable in lcpdat using the previously generated sequence
lcpdat$year <- years[as.numeric(substr(rownames(lcpdat), start=6, stop=6))]

# Set some parameters of the trellis display
trellis.par.set(list(strip.background = list(col="gray90", "gray90")))
# Draw the figure
xyplot(mean ~ status|factor(year) * issue, data=lcpdat, as.table=T, xlab="Freedom Status", par.strip.text=list(cex=.7), 
	ylab="Mean Value of Observed Variables", panel=function(x,y,subscripts){
	panel.segments(as.numeric(x),lcpdat[subscripts, "lower"], as.numeric(x), lcpdat[subscripts, "upper"])
	panel.segments(as.numeric(x)-.05, y, as.numeric(x)+.05,y, col="black")
})


