################################################################################
# Replication file for 'Stability and Change in the Freedom House Political    #
#    Rights and Civil Liberties Measures' 						               #
#                                                                              #
# This file creates the data and initial values to be used in the JAGS models  #
# This file must be run in R before the JAGS models can be run.                #
################################################################################

# TABLE OF CONTENTS (line numbers of this file indicated) #

###### Print Manuscript ###### 
# Load packaged and read in data ............... 20-40
# Prepare data for factor model ................ 42-69
# Write initial values for factor model ........ 71-130

###### On-line Appendix ###### 
# Prepare data for latent class model .......... 132-140
# Write initial values for latent class model .. 142-170 

#################################
## Load packages and read data ##
#################################

# Set working directory - should be where all of the replication files are placed
setwd("~/Desktop/rep")
# Load required libraries, most of these required for the post-processing as well
library(runjags)
library(gtools)
library(coda)
library(car)
library(grid)

## Read in raw data 
fh <- read.csv("jpr_replication.csv")

# Re-order data to be increasing in time then consecutive-integer country-code
fh <- fh[order(fh$time, fh$ccode), ]

# Create a press freedom factor variable to be used in the post-processing
fh$fhpf <- factor(fh$fhpf_num, levels=c(1,2,3), labels=c("Free", "Partly Free", "Not Free"))

#######################################
## Prepare data for the factor model ##
#######################################

# Pull out only the political rights variables and put them in a list where each element is one year's worth of data
polrts <- split(fh[,c("A", "B", "C")], fh$time)

# Subtract the column means (to center the data) in each element of the list
polrts <- lapply(polrts, function(x)sweep(x, 2, apply(x, 2, mean)))

# Use rbind to put together all of the centered matrices in one big matrix
polrts <- do.call(rbind, polrts)

# Make polrts an object of class matrix
polrts <- as.matrix(polrts)

# Set the row- and column-names attributes of polrts
rownames(polrts) <- colnames(polrts) <- NULL

# Do the same for civlibs that was just done for polrts (comments omitted)
civlibs <- split(fh[,c("D", "E", "F", "G")], fh$time)
civlibs <- do.call(rbind, lapply(civlibs, function(x)sweep(x, 2, apply(x, 2, mean))))
civlibs <- as.matrix(civlibs)
rownames(civlibs) <- colnames(civlibs) <- NULL

## Write out data for the factor model 
cat(dump.format(list(polrts = as.matrix(polrts), civlibs = as.matrix(civlibs), 
	time= fh$time, ccode=fh$ccode)), file="fh_prcl_data.txt")
	
################################################
## Create initial values for the Factor Model ##
################################################

# Function to create initial values for the factor model 
fac.inits <- function(seed = 1234){
	# Set random seet
	set.seed(seed)
	# Create the matrices of parameters (these are alpha, beta, delta and gamma from the manuscript)
	a.pr <- matrix(rnorm(12,0,10), ncol=3, nrow=4)
	b.pr <- matrix(rnorm(12,0,10), ncol=3, nrow=4)
	a.cl <- matrix(rnorm(16,0,10), ncol=4, nrow=4)
	b.cl <- matrix(rnorm(16,0,10), ncol=4, nrow=4)
	# Replace randomly generated missing values with NA for non-stochastic nodes
	b.pr[1,1] <- NA
	b.cl[1,1] <- NA
	b.pr[2,1] <- NA
	b.cl[2,1] <- NA
	b.pr[3,1] <- NA
	b.cl[3,1] <- NA
	b.pr[4,1] <- NA
	b.cl[4,1] <- NA
	a.pr[1,1] <- NA
	a.cl[1,1] <- NA
	a.pr[2,1] <- NA
	a.cl[2,1] <- NA
	a.pr[3,1] <- NA
	a.cl[3,1] <- NA
	a.pr[4,1] <- NA
	a.cl[4,1] <- NA
	
	# Draw the latent variable precisions from their appropriate gamma distributions
	tau.lpr <- rgamma(2, rate=1, shape=1)
	tau.lcl <- rgamma(2, rate=1, shape=1)
	
	# Draw the residual precisions for the PR and CL variables from their appropriate gamma distributions
	tau.cl <- matrix(rgamma(16, rate=1, shape=1), ncol=4, nrow=4)
	tau.pr <- matrix(rgamma(12, rate=1, shape=1), ncol=3, nrow=4)
	
	# Draw the first-period latent variable scores from their appropriate distribution
	latent.polrts <- latent.civlibs <- matrix(NA, ncol=4, nrow=193)
	latent.polrts[,1] <- rnorm(193, 0, 1/tau.lpr[1])
	latent.civlibs[,1] <- rnorm(193, 0, 1/tau.lcl[1])

	# Draw periods 2-4 latent variable scores from the right distributions
	latent.polrts[,2] <- latent.polrts[,1] + rnorm(193, 0, 1/tau.lpr[2])
	latent.polrts[,3] <- latent.polrts[,2] + rnorm(193, 0, 1/tau.lpr[2])	
	latent.polrts[,4] <- latent.polrts[,3] + rnorm(193, 0, 1/tau.lpr[2])
	latent.civlibs[,2] <- latent.civlibs[,1] + rnorm(193, 0, 1/tau.lcl[2])
	latent.civlibs[,3] <- latent.civlibs[,2] + rnorm(193, 0, 1/tau.lcl[2])
	latent.civlibs[,4] <- latent.civlibs[,3] + rnorm(193, 0, 1/tau.lcl[2])
	
	# Write all of the initial values to a list that will be returned by the function
	list(a.pr=a.pr, b.pr=b.pr, a.cl=a.cl, b.cl=b.cl, tau.pr=tau.pr, tau.cl=tau.cl, 
		tau.lpr=tau.lpr, tau.lcl=tau.lcl, latent.polrts = latent.polrts, latent.civlibs=latent.civlibs)
}

# Write out two chains worth of initial values, each using a different random seed
cat(dump.format(fac.inits(seed=1)), file="fh_prcl_inits1.txt")
cat(dump.format(fac.inits(seed=2)), file="fh_prcl_inits2.txt")

#########################################
## Prepare data for Latent Class Model ##
#########################################

# Pull out all sub-indicator scores and save them in a matrix
y <- as.matrix(fh[, c("A", "B", "C", "D", "E", "F", "G")])

# Write out data for latent class model 
cat(dump.format(list(y=y, time=fh$time)), file="fh_lc_data.txt", append=F)

######################################################
## Create initial values for the Latent Class Model ##
######################################################

# Function to create initial values for the latent class analysis
lc.inits <- function(seed=1234){
	# Set random seed
	set.seed(seed)
	# Use the rdirichlet function from the 'gtools' package to draw the initial values for p
	p <- rdirichlet(4, c(1,1,1))
	# Use the sample function to sample the initial values for freedom status using the p which was generated above
	free <- sapply(fh$time, function(x)sample(c(1,2,3), 1, prob=p[x, ]))
	# Set initial values for the means of each observed variable for each time-period and freedom status.
	g1a <- matrix(rnorm(12,0,10), ncol=4, nrow=3)
	g2a <- matrix(rnorm(12,0,10), ncol=4, nrow=3)
	g3a <- matrix(rnorm(12,0,10), ncol=4, nrow=3)
	g4a <- matrix(rnorm(12,0,10), ncol=4, nrow=3)
	g5a <- matrix(rnorm(12,0,10), ncol=4, nrow=3)
	g6a <- matrix(rnorm(12,0,10), ncol=4, nrow=3)
	g7a <- matrix(rnorm(12,0,10), ncol=4, nrow=3)
	# Set the initial values for the residual precisions from their appropriate distributions
	tau <- matrix(rgamma(28, rate=1, shape=1), ncol=7, nrow=4)
	# Write all of the initial values to a list that will be returned
	list(p=p, free=free, g1a=g1a, g2a=g2a, g3a=g3a, g4a=g4a, g5a=g5a, g6a=g6a, g7a=g7a, tau=tau)
}

# Write out two chains worth of initial values, each using a different random seed
cat(dump.format(lc.inits(seed=1)), file="fh_lc_inits1.txt")
cat(dump.format(lc.inits(seed=2)), file="fh_lc_inits2.txt")
